use std::str;
use crate::utils::{liveness_check};
use crate::clouds::defs::{
	Cloud,
	Provider,
	TskSrvc,
	App,
};
use reqenv::ReqEnv;
use crate::defs::{
	IsCritical,
	KloudCheckHome,
	SSHAccess,
	Cntrllr,
	CloudCheckGroup,
	CloudCheckItem
};
use crate::clouds::defs::{TsksrvcInfo,AppsrvcInfo,SrvcsHostInfo};

use crate::clouds::utils::{
	run_ssh_on_srvr,
	load_cloud_check_config,
};
use crate::clouds::on_req::{
	on_cloud_name_req_check,
};

use crate::monitor::get_cloud_monitor_info;

pub async fn parse_srvr_tsksrvcs(hostname: &str, sshaccess: &SSHAccess, tsksrvcs: &Vec<TskSrvc>,req_tsksrvcs: &str) -> Vec<TsksrvcInfo> {
  let mut tsksrvcs_info: Vec<TsksrvcInfo> = Vec::new();
	for tsk in req_tsksrvcs.split(",") {
		match format!("{}",&tsk).as_str() {
			"os" => {
				let os_name = String::from("os");
				tsksrvcs_info.push( TsksrvcInfo {
					name: format!("{}",&os_name),
					info: run_ssh_on_srvr(&hostname, &os_name, "", &sshaccess)
						.unwrap_or_else(|e| {
							eprintln!("run_ssh_on_srvr os: {}",e);
							String::from("")
						}),
					srvc: TskSrvc::default(),
				});
			},
			"floatip" => {
				let floatip_name = String::from("floatip");
				tsksrvcs_info.push( TsksrvcInfo {
					name: format!("{}",&floatip_name),
					info: run_ssh_on_srvr(&hostname, &floatip_name, "", &sshaccess)
						.unwrap_or_else(|e| {
							eprintln!("run_ssh_on_srvr floatip: {}",e);
							String::from("")
						}),
					srvc: TskSrvc::default(),
				});
			},
			"kubernetes_pods" => {
				let k8_name = String::from("kubernetes");
				tsksrvcs_info.push(TsksrvcInfo {
					name: format!("{}_pods",&k8_name),
					info: run_ssh_on_srvr(&hostname, &k8_name, "pods", &sshaccess)
						.unwrap_or_else(|e| {
							eprintln!("run_ssh_on_srvr kubernetes_pods: {}",e);
							String::from("")
						}),
					srvc: TskSrvc::default(),
					});
			},
			_ => { continue; }
		};
	}
	for tsksrvc in tsksrvcs.iter() {
		match format!("{}",&tsksrvc.name).as_str() {
			"pause" => continue,
			"scale" => continue,
			"systemfix" => continue,
			"os" => continue,
			_ => {
				let name = format!("{}",&tsksrvc.name);
				if req_tsksrvcs == "all" || req_tsksrvcs.contains(&name) {
					// TODO ssh &srv.hostname to get &name in "yaml"
					//println!("{} {} {}",&hostname,sshaccess.user,&tksrvc.name);
					tsksrvcs_info.push(TsksrvcInfo {
						name: format!("{}",&tsksrvc.name),
						info: run_ssh_on_srvr(&hostname, &name, "", &sshaccess)
						.unwrap_or_else(|e| {
							eprintln!("run_ssh_on_srvr for {}: {}",&tsksrvc.name,e);
							String::from("")
						}),
						srvc: tsksrvc.to_owned(),
					});
				}
			}
		};
	}
	tsksrvcs_info.to_owned()
}
pub async fn liveness_srvr_tsksrvcs(source: &str, cntrllrs: &Vec<Cntrllr>, tsksrvcs: &Vec<TskSrvc>, req_tsksrvc: &str) -> Vec<TsksrvcInfo> {
  let mut tsksrvcs_info: Vec<TsksrvcInfo> = Vec::new();
	let debug=envmnt::get_isize("DEBUG",0);
	for tsksrvc in tsksrvcs.iter() {
		match format!("{}",&tsksrvc.name).as_str() {
			"pause" => continue,
			"scale" => continue,
			"systemfix" => continue,
			_ => {
				let name = format!("{}",&tsksrvc.name);
				if tsksrvc.liveness.is_empty() || (req_tsksrvc != "" && !req_tsksrvc.contains(&name)) {
					continue;
				}
				let serverstring = format!("{}",&tsksrvc.liveness);
				if debug > 2 {
					println!("livenes: {} -> {}",&tsksrvc.name,&serverstring);
				}
				let res_info = match liveness_check(&source,&cntrllrs,&serverstring,"",&name).await {
					Ok(_) => "ok",
					Err(e) => {
						if tsksrvc.critical == IsCritical::yes {
						 	let monitor_name = "WUJI_MONITOR";
					    println!("{} critical livenes: {} -> {}",envmnt::get_or(&monitor_name,""),&tsksrvc.name,&serverstring);
						}
						if debug > 0 {
							eprint!("liveness_check error: {}",e);
						}
						"err"
					},
				};
				// println!("{} info: {}",&tsksrvc.name,&res_info);
				tsksrvcs_info.push(TsksrvcInfo {
					name: format!("{}",&tsksrvc.name),
					info: serde_yaml::from_str(res_info).unwrap_or_else(|e| {
						eprintln!("Serde liveness Error: {} {} -> {}",&source,&serverstring,e);
					  String::from("")
					}),
					srvc: tsksrvc.to_owned(),
				});
		  },
		}
	}
	tsksrvcs_info.to_owned()
}
pub async fn parse_srvr_appsrvcs(hostname: &str, sshaccess: &SSHAccess, appsrvcs: &Vec<App>,req_tsksrvcs: &str) -> Vec<AppsrvcInfo> {
  let mut appsrvcs_info: Vec<AppsrvcInfo> = Vec::new();
	for appsrvc in appsrvcs.iter() {
		let name = format!("{}",&appsrvc.name);
		if req_tsksrvcs == "all" || req_tsksrvcs.contains(&name) {
			// TODO ssh &srv.hostname to get &name in "yaml"
			//println!("{} {} {}",&hostname,sshaccess.user,&tksrvc.name);
			appsrvcs_info.push(AppsrvcInfo {
				name: format!("{}",&appsrvc.name),
				info: run_ssh_on_srvr(&hostname, &name, "", &sshaccess)
				.unwrap_or_else(|e| {
					eprintln!("run_ssh_on_srvr for {}: {}",&appsrvc.name,e);
					String::from("")
				}),
				srvc: appsrvc.to_owned(),
			});
		}
	}
	appsrvcs_info.to_owned()
}
pub async fn liveness_srvr_appsrvcs(source: &str, cntrllrs: &Vec<Cntrllr>, appsrvcs: &Vec<App>, req_tsksrvc: &str) -> Vec<AppsrvcInfo> {
  let mut appsrvcs_info: Vec<AppsrvcInfo> = Vec::new();
	let debug=envmnt::get_isize("DEBUG",0);
	for appsrvc in appsrvcs.iter() {
		// match format!("{}",&appsrvc.name).as_str() {
		// 	"pause" => continue,
		// 	"scale" => continue,
		// 	"systemfix" => continue,
		// 	_ => {
				let name = format!("{}",&appsrvc.name);
				if appsrvc.liveness.is_empty() || (req_tsksrvc != "" && !req_tsksrvc.contains(&name)) {
					continue;
				}
				let serverstring = format!("{}",&appsrvc.liveness);
				let live_req = format!("{}",&appsrvc.req);
				if debug > 2 {
					println!("livenes: {} -> {}",&appsrvc.name,&serverstring);
				}
				let res_info = match liveness_check(&source,&cntrllrs,&serverstring,&live_req,&name).await {
					Ok(_) => "ok",
					Err(e) => {
						if appsrvc.critical == IsCritical::yes {
						 	let monitor_name = "WUJI_MONITOR";
					    println!("{} critical livenes: {} -> {}",envmnt::get_or(&monitor_name,""),&appsrvc.name,&serverstring);
						}
						if debug > 0 {
							eprint!("liveness_check error: {}",e);
						}
						"err"
					},
				};
				// println!("{} info: {}",&appsrvc.name,&res_info);
				appsrvcs_info.push(AppsrvcInfo {
					name: format!("{}",&appsrvc.name),
					info: serde_yaml::from_str(res_info).unwrap_or_else(|e| {
						eprintln!("Serde liveness Error: {} {} -> {}",&source,&serverstring,e);
					  String::from("")
					}),
					srvc: appsrvc.to_owned(),
				});
		//   },
		// }
	}
	appsrvcs_info.to_owned()
}

pub async fn create_cloud_check(req_tsksrvcs: &str,reqenv: &ReqEnv,entries: Vec<String>,mut cloud: Cloud) -> Vec<KloudCheckHome> {
	let	mut cfg_entries: Vec<KloudCheckHome> = Vec::new();
	for entry in entries.iter() {
		let (mut cfg,_provider,cfg_data) = load_cloud_check_config(&mut cloud, &entry).await
			.unwrap_or_else(|e| {
				eprintln!("load_cloud_check_config: {}",e);
				(KloudCheckHome::default(),Provider::default(),String::from(""))
		});
		if cfg.name.is_empty() || cfg_data.is_empty() {
			let result = format!("Errors loading {}",&entry);
			if envmnt::get_isize("DEBUG",0) > 0 {
			  println!("{}",&result);
			}
			continue;
		}
		if req_tsksrvcs.contains("monitor") {
			cfg.monitor_info = Some(get_cloud_monitor_info(&mut cloud, &entry).await
					.unwrap_or_else(|e| {
						eprintln!("Error {} monitor_info {} -> {}",&entry,&cloud.env.monitor_run,e);
						String::from("")
					}));
		}
		if req_tsksrvcs.contains("liveness") || req_tsksrvcs.contains("apps") || req_tsksrvcs.contains("status") {
		  let mut groups: Vec<CloudCheckGroup> = Vec::new();
			// cfg.groups = cfg.groups.map(|grp| grp.with_resources(grp.path.to_owned())).collect();
			for grp in cfg.groups.iter() { 
		    let mut items: Vec<CloudCheckItem> = Vec::new();
			  for itm in grp.items.iter() { 
					let liveness: Vec<SrvcsHostInfo>;
					if req_tsksrvcs.contains("liveness") {
					 	liveness = on_cloud_name_req_check("liveness",&cloud,&reqenv,"","",&itm.path).await;
					} else if req_tsksrvcs.contains("apps") {
					 	liveness = on_cloud_name_req_check("apps",&cloud,&reqenv,"","",&itm.path).await;
					} else if req_tsksrvcs.contains("status") {
					 	liveness = on_cloud_name_req_check("status",&cloud,&reqenv,"","",&itm.path).await;
					} else {
						liveness = itm.liveness.to_owned();
					}
					items.push(CloudCheckItem {
						name: itm.name.to_owned(),
						info: itm.info.to_owned(),
						path: itm.path.to_owned(),
						liveness,
						critical: itm.critical.to_owned(),
					});
				}
				groups.push(CloudCheckGroup {
					name: grp.name.to_owned(),
					info: grp.info.to_owned(),
					path: grp.path.to_owned(),
					// TODO check this for group
					liveness: grp.liveness.to_owned(),
					items,
				});
			}
			cfg.groups=groups;
		} 
		cfg_entries.push(cfg.to_owned());
	}
	cfg_entries
}